/*
 * Copyright (c) 2011.
 * All rights reserved.
 *
 * This file is part of the uIP TCP/IP stack
 *
 * Modified by Jaroslav Fojtik 2010 - 2018
 */

#include "uip.h"
#include "ftpd.h"
#include "../misc/memb.h"
#include "../misc/tel_tools.h"
#include "ftpshell.h"


#include <string.h>


#define FTP_PORT  21		/**< TCP/IP port for FTP transfer. */


/* FatFs declarations */
#include "ff.h"	
#ifdef FF_DEFINED
void Fclose(FIL *F);
#endif

#include "FreeRTOS.h"
#include "task.h"


/* #define	DEBUG_EVENT		0x2000
#define	ERROR_EVENT		0xA000
extern void LogEvent(u16_t level, const char *text, ...); */


/*---------------------------------------------------------------------------*/
static int ftp_rexmit(ftpd_state *s)
{
  unsigned char rxNumSent = 0;		// @TODO: Tohle by melo preposlat data jen do "numsent".
  return senddata(s->lines, &rxNumSent);
}


/*---------------------------------------------------------------------------*/
static void ftp_senddata(ftpd_state *s)
{
	/* check status of connection #2 */
  if(s->conn2)
  {
    switch(((ftpd_state2 *)&(s->conn2->appstate))->shell2state)
    {
		// Completed transfer waiting for report
      case 0x81:
      case 0x82:
      case 0x84:
		 shell_output1(s->lines, "226 Transfer complete.",s->numsent); //348 bytes in 0.00 sec. (339.844 Kb/s)
                 ((ftpd_state2 *)&(s->conn2->appstate))->shell2state |= 0x20;  // 0x81 -> 0xA1 ...
                 ((ftpd_state2 *)&(s->conn2->appstate))->timer_tck = xTaskGetTickCount();
                 ((ftpd_state2 *)&(s->conn2->appstate))->state2 = STATE_PRE_CLOSE;
		 break;		// Do not discard connection immediatelly.

		// Transfer already reported, but rexmit might be emitted.
      case 0xA1:
      case 0xA2:
      case 0xA4:
                 if((unsigned)(xTaskGetTickCount()-((ftpd_state2 *)&(s->conn2->appstate))->timer_tck) > ms2tickCount(2000))
                     goto FinishConn2;
                 break;

      case 0xC1:
      case 0xC2:
      case 0xC4: shell_output1(s->lines, "550 Permission denied.", s->numsent);
FinishConn2:	 ((ftpd_state2 *)&(s->conn2->appstate))->shell2state = CON2_NONE;
		 ((ftpd_state2 *)&(s->conn2->appstate))->state2 = STATE_CLOSE;
		 if(((ftpd_state2 *)&(s->conn2->appstate))->file)
		 {
		   Fclose(((ftpd_state2 *)&(s->conn2->appstate))->file);
		   ((ftpd_state2 *)&(s->conn2->appstate))->file = NULL;
                 }
    		 s->conn2 = NULL;
		 break;
    }
  }
  	/* Push all cached data into a packet. */
  senddata(s->lines,&s->numsent);
}


/*---------------------------------------------------------------------------*/
static void ftp_close(ftpd_state *s)
{
	/* Kill 2nd connection. */
  if(s->conn2)
  {
    ((ftpd_state2 *)&(s->conn2->appstate))->state2 = STATE_CLOSE;
    s->conn2 = NULL;    
  }
 
  close_lines(s->lines); 

  if(s->Directory)
  {
    free(s->Directory);
    s->Directory = NULL;
  }
  if(s->Filename)
  {
    free(s->Filename);
    s->Filename = NULL;
  }
}


/*---------------------------------------------------------------------------*/

/** New data has been received through FTP connection. */
static void ftp_newdata(ftpd_state *s)
{
  u16_t len;
  char *dataptr;
  
  len = uip_datalen();
  dataptr = (char *)uip_appdata;
  
  while(len>0 && s->bufptr<sizeof(s->buf))
  {    
    --len;

    IAC(s->lines,*dataptr,&s->TS, s->numsent);

    if(s->TS.state==STATE_NORMAL)
      if(store_char(s->buf,&s->bufptr,*dataptr,sizeof(s->buf)))
        {ftp_shell_input(s, s->buf); s->bufptr = 0;}    

    ++dataptr;
  }  
}


/*---------------------------------------------------------------------------*/

/** Callback from uIP stack. */
static void ftpd_appcall(void)
{
ftpd_state *s = (ftpd_state *)&(uip_conn->appstate);

  if(uip_connected())			/* Connection is not established yet. */
  {
    /* tcp_markconn(uip_conn, &s); */    
    memset(s->lines, 0, sizeof(s->lines));
    s->Directory = NULL;
    s->Filename = NULL;
    s->bufptr = 0;
    s->TS.state = STATE_NORMAL;
    s->TS.InteligentTerm = 0;
    s->TS.ActiveCommand = 0xFF;		// No Cmd
    s->conn2 = NULL;

    ftp_shell_start(s);
  }

  if(s->TS.state == STATE_CLOSE)		/* The record signalises that connedtion should be closed. */
  {
    if(s->conn2!=NULL)
    {
      ((ftpd_state2 *)&(s->conn2->appstate))->connOrig = NULL;
      s->conn2 = NULL;
    }
    s->TS.state = STATE_NORMAL;
    uip_close();
    return;
  }
  
  if(uip_closed() || uip_aborted() || uip_timedout())	/* uIP request to close. */ 
  {
    ftp_close(s);
  }
  
  if(uip_acked()) 
  {
    acked(s->lines,&s->numsent,FTPD_CONF_NUMLINES);
  }
  
  if(uip_newdata()) 			/* New data arrived. */
  {
    ftp_newdata(s);
  }
  
  if(uip_rexmit())
  {
    if(ftp_rexmit(s)>0) return;
  }

  if(uip_rexmit() || uip_newdata() || uip_connected() || uip_poll())   /* Something to send? */
  {
    ftp_senddata(s);
  }
}


/*---------------------------------------------------------------------------*/

/** Initialisation of FTP module. */
void ftpd_init(void)
{
  uip_listen(HTONS(FTP_PORT),ftpd_appcall);
  memb_init(&linemem);
}
