/*
 * Copyright (c) 2003 - 2016, Jaroslav Fojtik.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above
 *    copyright notice, this list of conditions and the following
 *    disclaimer in the documentation and/or other materials provided
 *    with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
 * OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
 * GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This file is part of the uIP TCP/IP stack
 *
 * $Id: ftpd.h,v 1.2 2006/06/07 09:43:54 adam Exp $
 *
 */
#ifndef __FTPD_H__
#define __FTPD_H__

#include "uipopt.h"
#include "../misc/tel_tools.h"

#ifdef __cplusplus
extern "C" {
#endif


#define FTPD_CONF_LINELEN_IN    40
#define FTPD_CONF_LINELEN_OUT   CONF_LINELEN_OUT
#define FTPD_CONF_NUMLINES	CONF_NUMLINES

/** Status of FTP connection, command line. */
typedef struct
{
  struct uip_conn *conn2;	/**< 2nd communication object. */
  u8_t numsent;			/**< Number of 'lines' sent during last transmition.*/
  u8_t type;			/**< 0 - binary; 1 - ASCII */
  u8_t shellstate;		/**< ::FTP_NOT_LOGGED -> ::FTP_USERNAME_OK -> ::FTP_USER_OK */
  u8_t UserIDX;			/**< Index to a table of users. */ 
  u8_t IP0,IP1,IP2,IP3;		/**< IP address of a client. */
  u8_t Port0,Port1;  		/**< port of a client. */
  char *Directory;		/**< Current directory of a session. */  
  char *Filename;  
  char *lines[FTPD_CONF_NUMLINES]; /**< Output transmition buffer. */
  char buf[FTPD_CONF_LINELEN_IN]; /**< Buffer for data receiving. */
  char bufptr;			/**< Pointer into data receiving buffer. */
  term_state TS;                /**< Terminal state including ::STATE_CLOSE; ::STATE_NORMAL */
} ftpd_state;


/** Status of FTP connection 2nd data line. */
typedef struct {
  struct uip_conn *connOrig;
  u8_t state2;
  u8_t shell2state;
  char buf[FTPD_CONF_LINELEN_IN];  /**< Buffer that contains one piece of data, allows to rexmit when data lost. */
#if defined(_FATFS) || defined(FF_DEFINED)
  DIR  *dirs;
  FIL  *file;
#else
  void  *dirs;
  void  *file;
#endif
  u32_t StartSeq;		/**< Transmission sequence counter when file download is started. */
  u16_t retry_size;		/**< Amount of write SD retry fragments. */
  unsigned timer_tck;
  void *retryPtr;		/**< Pointer to fragments to be written on SD. */
} ftpd_state2;


/** FTP line #2 status. */
typedef enum
{
  CON2_NONE	= 0,
  CON2_RNLST    = 1,
  CON2_RGET     = 2,
  CON2_RPUT     = 3,
  CON2_RLIST    = 4,

  CON2_RNLST_OK   = 0x81,
  CON2_RGET_OK    = 0x82,
  CON2_RPUT_OK    = 0x83,
  CON2_RLIST_OK   = 0x84,

  CON2_RNLST_FIN  = 0xA1,
  CON2_RGET_FIN   = 0xA2,

  CON2_RNLST_FAIL = 0xC1,
  CON2_RGET_FAIL  = 0xC2,
  CON2_RPUT_FAIL =  0xC3,
  CON2_RLIST_FAIL = 0xC4
} CON2_STATUS;

#define STATE_PRE_CLOSE  11


/** typedef ftpd_state uip_tcp_appstate_t; */

#ifdef _FATFS
FIL *Fopen(const char *FileName);
FIL *FopenW(const char *FileName);
FIL *FopenA(const char *FileName);
void Fclose(FIL *F);
#endif
int FreeMem(void);


void ftpd_init(void);


#ifdef __cplusplus
 }
#endif

#endif /* __FTPD_H__ */
