/**
 * \file
 * Interface for the Contiki shell.
 * \author Adam Dunkels <adam@dunkels.com>
 *
 * Some of the functions declared in this file must be implemented as
 * a shell back-end in the architecture specific files of a Contiki
 * port.
 */

#ifndef __FTP_SHELL_H__
#define __FTP_SHELL_H__


/**
 * Initialize the shell.
 *
 * Called when the shell front-end process starts. This function may
 * be used to start listening for signals.
 */
void ftp_shell_init(void);

/**
 * Start the shell back-end.
 *
 * Called by the front-end when a new shell is started.
 */
void ftp_shell_start(ftpd_state *s);

/**
 * Process a shell command.
 *
 * This function will be called by the shell GUI / telnet server whan
 * a command has been entered that should be processed by the shell
 * back-end.
 *
 * \param command The command to be processed.
 */
void ftp_shell_input(ftpd_state *s, char *command);


/**
 * Print a string to the shell window.
 *
 * This function is implemented by the shell GUI / telnet server and
 * can be called by the shell back-end to output a string in the
 * shell window. The string is automatically appended with a linebreak.
 *
 * \param str1 The first half of the string to be output.
 * \param str2 The second half of the string to be output.
 */
void ftp_shell_output(ftpd_state *s, const char *str1, const char *str2);

/**
 * Print a prompt to the shell window.
 *
 * This function can be used by the shell back-end to print out a
 * prompt to the shell window.
 *
 * \param prompt The prompt to be printed.
 *
 */
void ftp_shell_prompt(ftpd_state *s, const char *prompt);

#endif /* __FTP_SHELL_H__ */
