/******************************************************************************
* *
* evolving systems consulting *
* http://www.evolvsys.cz *
* *
* ALL RIGHTS RESERVED *
* *
******************************************************************************/
/** @file
 * Telnet shell skeleton - copy it into your project, and fill missing features. */

#include <string.h>
#include <ctype.h>
#include <stdio.h>

	/* uIP stuff. */
#include "uipopt.h"
#include "uip.h"
#include "uip_arp.h"
#include "../uip/telnetd/telnetd.h"
#include "../uip/telnetd/shell.h"
//#include "../uip/misc/UserList.h"

#include "logger.h"


/** Telnet structure for a command. */
typedef struct {
  const char *commandstr;
  const char *description;
  void (* pfunc)(telnetd_state *s, char *str);
} ptentry;


static const char *SHELL_PROMPT  = "uIP> ";


/*---------------------------------------------------------------------------*/
static void parse(telnetd_state *s, register char *str, const ptentry *p)
{
int i;
  if(p==NULL) return;

  while(isspace(*str))
  {    
    str++;
  }

  while(p->commandstr != NULL)
  {
    i = strlen(p->commandstr);
    if(strncmp(p->commandstr, str, strlen(p->commandstr)) == 0) 
    {
      str += i;
      while(isspace(*str))
	str++;
      break;
    }
    ++p;
  }
  p->pfunc(s,str);
}


/** Unknown command response. */
static void unknown(telnetd_state *s, char *str)
{
  if(strlen(str) > 0) {
    shell_output(s->lines,"Unknown command: ", str);
  }
}


/** List of ARP table to user. */
static void arp_list(telnetd_state *s, char *str)
{
char buffer[3*4+3 + 1 + 2*6+5 + 3];
arp_entry *tabptr;

  shell_output1(s->lines,"ARP table listing:");
  
  for(tabptr=arp_table; tabptr<arp_table+UIP_ARPTAB_SIZE; ++tabptr)
  {
    if(tabptr->ipaddr[0]==0 && tabptr->ipaddr[1]==0) continue;

    sprintf(buffer, "%u.%u.%u.%u %2.2X:%2.2X:%2.2X:%2.2X:%2.2X:%2.2X",
        uip_ipaddr1(tabptr->ipaddr), uip_ipaddr2(tabptr->ipaddr),
	uip_ipaddr3(tabptr->ipaddr), uip_ipaddr4(tabptr->ipaddr),
	tabptr->ethaddr.addr[0], tabptr->ethaddr.addr[1], tabptr->ethaddr.addr[2],
	tabptr->ethaddr.addr[3], tabptr->ethaddr.addr[4], tabptr->ethaddr.addr[5]);
	
    shell_output1(s->lines, buffer);
  }
}


extern const char *states[];
static void conn_list(telnetd_state *s, char *str)
{
struct uip_conn *conn;
int i;
char *line;

  for(i=0; i<UIP_CONNS; i++)
  {
    conn = &uip_conns[i];
    if(conn->lport==0) continue;

    line = tel_alloc_line();
    if(line)
    {      
      snprintf(line, CONF_LINELEN_OUT,
		 "%d %u.%u.%u.%u:%u %s %u %u %c %c\r\n",
		 htons(conn->lport),
		 htons(conn->ripaddr[0]) >> 8,
		 htons(conn->ripaddr[0]) & 0xff,
		 htons(conn->ripaddr[1]) >> 8,
		 htons(conn->ripaddr[1]) & 0xff,
		 htons(conn->rport),
		 states[conn->tcpstateflags & UIP_TS_MASK],
		 conn->nrtx,
		 conn->timer,
		 (uip_outstanding(conn))? '*':' ',
		 (uip_stopped(conn))? '!':' ');
      sendline(s->lines, line, TELNETD_NUMLINES);
    }
  }
}


static void help(telnetd_state *s, char *str);


/*---------------------------------------------------------------------------*/
static const ptentry parsetab[] =
  {//{"stats","   - show network statistics", help},
   {"conn","    - show TCP connections", conn_list},
   {"help",     ", ? - show help", help},
   {"exit",     "    - exit shell", shell_quit},
   {"arp",      "    - list ARP table", arp_list},
   {"?",NULL, help},

   /* Default action */
   {NULL, NULL, unknown}};



/*---------------------------------------------------------------------------*/
static void help(telnetd_state *s, char *str)
{
const ptentry *p;
  shell_output1(s->lines, "Available commands:");

  for(p=parsetab; p->commandstr!=NULL; p++)
  {
    shell_output(s->lines, p->commandstr, p->description);
  }
}



/*---------------------------------------------------------------------------*/
/** This callback is called during full initialisation. */
void shell_init(void)
{  
}


/*---------------------------------------------------------------------------*/
void shell_start(telnetd_state *s)
{
  LogEvent(INFO_EVENT, "Telnet shell granted to %u.%u.%u.%u",
	 uip_ipaddr1(uip_conn->ripaddr), uip_ipaddr2(uip_conn->ripaddr),
	 uip_ipaddr3(uip_conn->ripaddr), uip_ipaddr4(uip_conn->ripaddr));

  shell_output1(s->lines, "uIP command shell");
  shell_prompt(s, "Username: ");

  s->shellstate = TEL_GET_USERNAME;
}


/** Obtain username from user. */
static void tel_username(telnetd_state *s, char *str)
{
//int i;

  if(str && *str!=0)
  { 
    /*  This is a template only:
              Scan your own table of users. When user is found, set  s->shellstate = TEL_USERNAME_OK
               otherwise s->shellstate = TEL_NOT_LOGGED; */

    /* if(i>=0) */
      s->shellstate = TEL_USERNAME_OK;
    /* else    
                 s->shellstate = TEL_NOT_LOGGED; */
  }
}


/** Obtain password from user. */
static void tel_password(telnetd_state *s, char *str)
{
  if(s->shellstate>=TEL_USERNAME_OK && str!=NULL)
  {
    // TODO: dopsat!    
    //if(UserList!=NULL && s->UserIDX<MAX_USERS)
    //{
    //  if(strncmp(str,UserList[s->UserIDX].Password,PASSWORD_SIZE)) goto REJECT_PASSWOD;
      /* str = UserList[s->UserIDX].UserName; */
    //}
    /* else		Do not check username/password when userlist is empty.
      str = "xxx";	 */

    s->shellstate = TEL_USER_OK;
  }
  else
  {
//REJECT_PASSWOD:
    s->shellstate = TEL_NOT_LOGGED;
  }
}


/*---------------------------------------------------------------------------*/
void shell_input(telnetd_state *s, char *cmd)
{  
  switch(s->shellstate)
  {
    case TEL_USERNAME_OK: 
                tel_password(s,cmd);
                if(s->shellstate==TEL_USER_OK)
	        {
                  shell_output1(s->lines, "Type '?' and return for help");
                  shell_prompt(s,SHELL_PROMPT); 
                  break;
                }
				/* continue on next label */
    default:	s->shellstate = TEL_NOT_LOGGED;
				/* continue on next label */  		
    case TEL_GET_USERNAME:		
                tel_username(s,cmd);
                if(s->shellstate==TEL_USERNAME_OK)
                {
                  shell_prompt(s,"Password: ");
	          break;
                }
				/* continue on next label */
    case TEL_NOT_LOGGED:
	        shell_prompt(s,"Username: "); 
                s->shellstate = TEL_GET_USERNAME;
		break;

    case TEL_USER_OK:
                if(cmd)
                {    
                  parse(s, cmd, parsetab);     
                }
	        shell_prompt(s,SHELL_PROMPT); 
	        break;
  }
}
/*---------------------------------------------------------------------------*/
