/*
	FreeRTOS.org V4.5.0 - Copyright (C) 2003-2007 Richard Barry.
*/
/* Standard includes. */
#include <string.h>
#include <stdlib.h>

/* Scheduler includes. */
#include "FreeRTOS.h"
#include "task.h"
#include "semphr.h"

/* uip includes. */
#include "hw_types.h"
#include "uip.h"
#include "uip_arp.h"
#include "timer.h"
#include "clock-arch.h"
#include "../dhcpc/dhcpc.h"

/* Demo includes. */
#include "emac.h"

#include "logger.h"


#if UIP_LOGGING == 1
void uip_log(char *msg)
{
  LogEvent(WARNING_EVENT, "uIP %s", msg);
}
#endif


unsigned char BadCrcReceived = 0;

extern const struct uip_eth_addr broadcast_ethaddr;


#if UIP_FIXEDADDR==0
 #if UIP_UDP==0
  #error DHCP client requires UDP to be compiled!
 #endif
#endif


/* The buffer used by the uIP stack.  In this case the pointer is used to
point to one of the Rx buffers. */
unsigned portCHAR *uip_buf;
//#warning uip_buf je globalni promenna a nehlida se prepisovani !!!!!!!


#ifdef emacCONF_NetWait_ms
 #define NetWait_ms emacCONF_NetWait_ms
#else
 #define NetWait_ms		20		///< peroidic waiting period for some network job.
#endif

#ifdef emacCONF_NetFastWait_ms
 #define NetFastWait_ms emacCONF_NetFastWait_ms
#else
 #define NetFastWait_ms		6
#endif


volatile char	UDP_EXTRA_DATA = 0;	/**< If this flag is set, the next UDP pool is done in shorter interval. */
volatile char	IP_EXTRA_DATA = 0;	/**< If this flag is set, the next IP pool is done in shorter interval. */
char EthNeedsReset = 1;			/**< Write here 1 to force RESET ETH peripheral. */

#define ETH_FAIL_INC    50 
#define MAX_ETH_FAILS	ETH_FAIL_INC*4	/**< Treshold value that causes ETH to RESET. Must be less than 256. */

/*-----------------------------------------------------------*/

/* USE DHCP for compilation into emulator. */
#if 0
 #ifndef FAKE_SIM_TIMER
  #ifdef UIP_FIXEDADDR
   #undef UIP_FIXEDADDR
  #endif
  #define UIP_FIXEDADDR 1
  #warning Forced UIP_FIXEDADDR
 #endif
#endif




/* IP address configuration. */
#define uipIP_ADDR0 	192
#define uipIP_ADDR1 	168
#define uipIP_ADDR2 	1
#define uipIP_ADDR3 	123

#define uipIP_MASK0	255
#define uipIP_MASK1	255
#define uipIP_MASK2	255
#define uipIP_MASK3	0

#define uipIP_GW0 	192
#define uipIP_GW1 	168
#define uipIP_GW2 	1
#define uipIP_GW3 	200

char DHCP_FLAG = 1;


/* How long to wait before attempting to connect the MAC again. */
#define uipINIT_WAITms		100		/* 100ms */

/* Shortcut to the header within the Rx buffer. */
#define xHeader ((struct uip_eth_hdr *) &uip_buf[0])

/* Standard constant. */
#define uipTOTAL_FRAME_HEADER_SIZE	54

/*-----------------------------------------------------------*/

/** Send the uIP buffer to the MAC. */
inline static void prvENET_Send(void)
{
  //printf("T");
  vSendBufferToMAC(&uip_buf,uip_len);
}

/** Setup the MAC address in the MAC itself, and in the uIP stack. */
void prvSetMACAddress(void);

/** Port functions required by the uIP stack. */
void clock_init(void)
{
	/* This is done when the scheduler starts. */
}

clock_time_t clock_time(void);

/*-----------------------------------------------------------*/

/*-----------------------------------------------------------*/

/* Define clock functions here to avoid header file name clash between uIP
and the Luminary Micro driver library. */
clock_time_t clock_time( void )
{
  return xTaskGetTickCount();
}


/** Callback from DHCP. */
void dhcpc_configured(const dhcpc_state *s)
{
  LogEvent(INFO_EVENT, "IP is configured from DHCP %u.%u.%u.%u",
      uip_ipaddr1(s->ipaddr), uip_ipaddr2(s->ipaddr),
      uip_ipaddr3(s->ipaddr), uip_ipaddr4(s->ipaddr));

  uip_sethostaddr(&s->ipaddr);
  uip_setnetmask(&s->netmask);
  uip_setdraddr(&s->default_router);
}


int EnsurePacket(void)
{
  if(uip_buf!=NULL)
  {    
    return 1;
  }
      
  uip_buf = (unsigned char *)GetFreePkt(0);
  if(!uip_buf)
  {    
    EMAC_ErrorFlags |= EMAC_ERR_OVERLOAD;	// Amount of available packets exhausted.
    return 0;
  }
  uip_buf += 2;
  return 2;
}


// int Normal=0, Fast=0;


/** Main Emac task. */
void vuIP_Task(void *pvParameters)
{
portBASE_TYPE i;
struct timer periodic_timer, arp_timer;
DWORD TickWait;
#if UIP_FIXEDADDR!=0
uip_ipaddr_t xIPAddr;
#endif  

	/* Initialise the uIP stack. */
  timer_set(&periodic_timer, configTICK_RATE_HZ/3); /* call 3* per second */
  timer_set(&arp_timer, configTICK_RATE_HZ*10);     /* Call once per 10s */
  uip_init();
#ifdef __HTTPD_H__
  httpd_init();
#endif
#ifdef __TELNETD_H__
  telnetd_init();
#endif
#ifdef __FTPD_H__
  ftpd_init();
#endif  

  while(vInitEMAC() != pdPASS)
  {
    vTaskDelay(ms2tickCount(uipINIT_WAITms));
  }
  prvSetMACAddress();
  EthNeedsReset = 0;

  uip_buf = (unsigned char *)GetFreePkt(0);
  if(uip_buf) uip_buf += 2;

#if UIP_FIXEDADDR==0
  if(DHCP_FLAG)
  {
    dhcpc_init(uip_ethaddr.addr,6);
    dhcpc_request();
  }
  else  // jumps to LogEvent() for "UIP_FIXEDADDR==0"
#else
  uip_ipaddr(xIPAddr, uipIP_ADDR0, uipIP_ADDR1, uipIP_ADDR2, uipIP_ADDR3);
  uip_sethostaddr(xIPAddr);
  uip_ipaddr(xIPAddr, uipIP_MASK0, uipIP_MASK1, uipIP_MASK2, uipIP_MASK3);
  uip_setnetmask(xIPAddr);
  uip_ipaddr(xIPAddr, uipIP_GW0, uipIP_GW1, uipIP_GW2, uipIP_GW3);
  uip_setdraddr(xIPAddr);  
#endif
    LogEvent(INFO_EVENT, "Used static IP address %u.%u.%u.%u", 
	uip_ipaddr1(uip_hostaddr), uip_ipaddr2(uip_hostaddr),
	uip_ipaddr3(uip_hostaddr), uip_ipaddr4(uip_hostaddr));
	
#ifdef __DHCPD_H__
  i = DHCPD_uIP_Init();		/* DHCP demon start. */
  if(i!=0 && i!=-1)
    LogEvent(ERROR_EVENT, "DHCP daemon failed to initialise with error code %d.", i);
#endif

#ifdef HAVE_APP_uIP_INIT_CALLBACK
  App_uIP_InitCallback();	/* Callback after UDP initialisation. */
#endif

  while(1)
  {
    TickWait = ms2tickCount(NetWait_ms);
    while(!EthNeedsReset)
    {
		/* Is there received data ready to be processed? */
      uip_len = uiGetEMACRxData(&uip_buf, TickWait);
		
      if(uip_len > 0)
      {      
        //LogEvent(DEBUG_EVENT, "Packet received len=%d, type=%d", uip_len, xHeader->type);      

			/* Standard uIP loop taken from the uIP manual. */
        switch(HTONS(xHeader->type))
        {
	  case UIP_ETHTYPE_IP:
          {
	    uip_arp_ipin();

	  	/* MAC address check - accept only own packet or broadcast */
            if(memcmp(xHeader->dest.addr,uip_ethaddr.addr,6))
              if(memcmp(xHeader->dest.addr,broadcast_ethaddr.addr,6)) 
			break;

		// TEST code - remove me.
            #define BUF ((struct uip_tcpip_hdr *)&uip_buf[UIP_LLH_LEN])            
            if(uip_len < (((u16_t)(BUF->len[0]) << 8) + BUF->len[1]))
	    {
              LogEvent(WARNING_EVENT, "Bad received packet size - def: %d, len: %u:%u", 
                          (int)BadCrcReceived, 
                           uip_len, (((u16_t)(BUF->len[0]) << 8) + BUF->len[1]));
              goto BAD_PKT;
            }
            else
            {
              i = BUF->ipchksum;
              BUF->ipchksum = 0;
              if(i != (0xFFFF & ~(uip_ipchksum())))
              {
                  //if(BadCrcReceived==0)
               /* LogEvent(WARNING_EVENT, "Bad received packet CRC - def: %d, len: %u:%u", 
                          (int)BadCrcReceived,
                          uip_len, (((u16_t)(BUF->len[0]) << 8) + BUF->len[1])); */
BAD_PKT:
                if(BadCrcReceived > MAX_ETH_FAILS)  /* More corrupted packets received :( */
                {
                  EthNeedsReset = 1;		/* Enforce peripheral RESET. */
                }
                else
                  BadCrcReceived += ETH_FAIL_INC;
                break;
              }                                        
              else
              {
                if(BadCrcReceived>0)
                {
                  BadCrcReceived--;
                  LogEvent(DEBUG_EVENT, "PKT - def: %d, len: %u:%u",	    // TODO: debug code - comment it out
                          (int)BadCrcReceived,
                          uip_len, (((u16_t)(BUF->len[0]) << 8) + BUF->len[1]));                  
                }
              }
              BUF->ipchksum = i;
            }
            
		// ~TEST code - remove me.

	    uip_input();

		/* If the above function invocation resulted in data that
		should be sent out on the network, the global variable
		uip_len is set to a value > 0. */
	    if(uip_len > 0)
	    {
	      uip_arp_out();	/* Fill a proper MAC address. */
	      prvENET_Send();	  
	    }
            
	    break;
          }
          case UIP_ETHTYPE_ARP:
          {
	    uip_arp_arpin();
	   
		/* If the above function invocation resulted in data that
		should be sent out on the network, the global variable
		uip_len is set to a value > 0. */
	    if( uip_len > 0 )
	    {
	      prvENET_Send();
	    }
	    break;
          }
        }
      }

	/* Check timer expiration all the time. Otherwise high packet traffic can cause
 	   that packets are read all the time and this section would never be entered. */
      if(timer_expired(&periodic_timer))
      {
        timer_reset(&periodic_timer);

#if UIP_UDP
        UDP_EXTRA_DATA = 1;   
#endif	
        IP_EXTRA_DATA = 1;
			
        if(timer_expired(&arp_timer))	/* Call the ARP timer function every 10 seconds. */
        {
	  timer_reset(&arp_timer);
	  uip_arp_timer();
        }
      }    

#if UIP_UDP
      if(UDP_EXTRA_DATA)
      {
        UDP_EXTRA_DATA = 0;
        for(i=0; i<UIP_UDP_CONNS; i++)
        {
          if(EnsurePacket()==0)
          {
            UDP_EXTRA_DATA = 1;            
            break;
          }        
          uip_len = 0;				/* Signalise fresh packet */
          uip_udp_periodic(i);			/* Any callback might set a flag: UDP_EXTRA_DATA */
          if(uip_len > 0) 
          {
	    uip_arp_out();
            prvENET_Send();
          }
        }	
      }
#endif

      if(IP_EXTRA_DATA)
      {
        IP_EXTRA_DATA = 0;
        for(i=0; i<UIP_CONNS; i++)
        {
          if(EnsurePacket()==0)
          {
            IP_EXTRA_DATA = 1;	/* We must pool in the next period, no free packet available. */
            break;          
          }        
          uip_len = 0;		/* Signalise fresh packet */
          uip_periodic(i);
          /*uip_poll_conn(&uip_conns[i]); * If the above function invocation resulted in data that
				   should be sent out on the network, the global variable
				   uip_len is set to a value > 0. */
          if(uip_len > 0) 
          {
            //LogEvent(DEBUG_EVENT, "IP conn %d sends a packet len=%d F:%X" , i, uip_len, uip_flags);
            uip_arp_out();
            prvENET_Send();
          }
        }	
      }	
		/* We did not receive a packet, and there was no periodic
		processing to perform.  Block for a fixed period.  If a packet
		is received during this period we will be woken by the ISR
		giving us the Semaphore. */          
      if(IP_EXTRA_DATA || UDP_EXTRA_DATA) TickWait = NetFastWait_ms;
  	                             else TickWait = NetWait_ms;          // Renew wait time.

      /*if(IP_EXTRA_DATA || UDP_EXTRA_DATA) Fast++;
                                   else Normal++;
      if(Fast>100 || Normal>20)
      {
        LogEvent(INFO_EVENT, "uIP Interval F:%d N:%d", Fast, Normal);
        Fast = Normal = 0;
      } */

    }

    LogEvent(WARNING_EVENT, "ETH peripheral RESET enforced!");
    while(vInitEMAC() != pdPASS)
    {
      vTaskDelay(ms2tickCount(uipINIT_WAITms));
    }    
    prvSetMACAddress();
    EthNeedsReset = 0;
  }
}


/*-----------------------------------------------------------*/





